#ifndef TYPON_FUNDAMENTAL_BINARY_RING_HPP_INCLUDED
#define TYPON_FUNDAMENTAL_BINARY_RING_HPP_INCLUDED

#include <atomic>
#include <cstdint>
#include <type_traits>


namespace typon::fdt
{

  template <typename I>
    requires std::is_unsigned_v<I>
  struct binary_ring
  {
    const I _mask;

    binary_ring(std::uint_least8_t bits) noexcept
      : _mask((I(1) << bits) - 1)
    {}

    I size() noexcept
    {
      return _mask + 1;
    }

    friend I operator % (I x, binary_ring ring) noexcept
    {
      return x & ring._mask;
    }
  };

}


#endif //  TYPON_FUNDAMENTAL_BINARY_RING_HPP_INCLUDED
